@@ -19,21 +19,21 @@
 from __future__ import (absolute_import, division, print_function)
 __metaclass__ = type
 
-import os
-
 from ansible import constants as C
 from ansible.errors import AnsibleError, AnsibleParserError, AnsibleUndefinedVariable, AnsibleAssertionError
 from ansible.module_utils._text import to_native
-from ansible.module_utils.six import iteritems, string_types
+from ansible.module_utils.six import string_types
 from ansible.parsing.mod_args import ModuleArgsParser
 from ansible.parsing.yaml.objects import AnsibleBaseYAMLObject, AnsibleMapping
 from ansible.plugins.loader import lookup_loader
-from ansible.playbook.attribute import FieldAttribute
+from ansible.playbook.attribute import NonInheritableFieldAttribute
 from ansible.playbook.base import Base
 from ansible.playbook.block import Block
 from ansible.playbook.collectionsearch import CollectionSearch
 from ansible.playbook.conditional import Conditional
+from ansible.playbook.delegatable import Delegatable
 from ansible.playbook.loop_control import LoopControl
+from ansible.playbook.notifiable import Notifiable
 from ansible.playbook.role import Role
 from ansible.playbook.taggable import Taggable
 from ansible.utils.collection_loader import AnsibleCollectionConfig
@@ -45,7 +45,7 @@ __all__ = ['Task']
 display = Display()
 
 
-class Task(Base, Conditional, Taggable, CollectionSearch):
+class Task(Base, Conditional, Taggable, CollectionSearch, Notifiable, Delegatable):
 
     """
     A task is a language feature that represents a call to a module, with given arguments and other parameters.
@@ -65,39 +65,33 @@ class Task(Base, Conditional, Taggable, CollectionSearch):
     # might be possible to define others
 
     # NOTE: ONLY set defaults on task attributes that are not inheritable,
-    # inheritance is only triggered if the 'current value' is None,
+    # inheritance is only triggered if the 'current value' is Sentinel,
     # default can be set at play/top level object and inheritance will take it's course.
 
-    _args = FieldAttribute(isa='dict', default=dict)
-    _action = FieldAttribute(isa='string')
+    args = NonInheritableFieldAttribute(isa='dict', default=dict)
+    action = NonInheritableFieldAttribute(isa='string')
 
-    _async_val = FieldAttribute(isa='int', default=0, alias='async')
-    _changed_when = FieldAttribute(isa='list', default=list)
-    _delay = FieldAttribute(isa='int', default=5)
-    _delegate_to = FieldAttribute(isa='string')
-    _delegate_facts = FieldAttribute(isa='bool')
-    _failed_when = FieldAttribute(isa='list', default=list)
-    _loop = FieldAttribute()
-    _loop_control = FieldAttribute(isa='class', class_type=LoopControl, inherit=False)
-    _notify = FieldAttribute(isa='list')
-    _poll = FieldAttribute(isa='int', default=C.DEFAULT_POLL_INTERVAL)
-    _register = FieldAttribute(isa='string', static=True)
-    _retries = FieldAttribute(isa='int', default=3)
-    _until = FieldAttribute(isa='list', default=list)
+    async_val = NonInheritableFieldAttribute(isa='int', default=0, alias='async')
+    changed_when = NonInheritableFieldAttribute(isa='list', default=list)
+    delay = NonInheritableFieldAttribute(isa='int', default=5)
+    failed_when = NonInheritableFieldAttribute(isa='list', default=list)
+    loop = NonInheritableFieldAttribute(isa='list')
+    loop_control = NonInheritableFieldAttribute(isa='class', class_type=LoopControl, default=LoopControl)
+    poll = NonInheritableFieldAttribute(isa='int', default=C.DEFAULT_POLL_INTERVAL)
+    register = NonInheritableFieldAttribute(isa='string', static=True)
+    retries = NonInheritableFieldAttribute(isa='int', default=3)
+    until = NonInheritableFieldAttribute(isa='list', default=list)
 
     # deprecated, used to be loop and loop_args but loop has been repurposed
-    _loop_with = FieldAttribute(isa='string', private=True, inherit=False)
+    loop_with = NonInheritableFieldAttribute(isa='string', private=True)
 
     def __init__(self, block=None, role=None, task_include=None):
         ''' constructors a task, without the Task.load classmethod, it will be pretty blank '''
 
-        # This is a reference of all the candidate action names for transparent execution of module_defaults with redirected content
-        # This isn't a FieldAttribute to prevent it from being set via the playbook
-        self._ansible_internal_redirect_list = []
-
         self._role = role
         self._parent = None
         self.implicit = False
+        self.resolved_action = None
 
         if task_include:
             self._parent = task_include
@@ -106,16 +100,6 @@ class Task(Base, Conditional, Taggable, CollectionSearch):
 
         super(Task, self).__init__()
 
-    def get_path(self):
-        ''' return the absolute path of the task with its line number '''
-
-        path = ""
-        if hasattr(self, '_ds') and hasattr(self._ds, '_data_source') and hasattr(self._ds, '_line_number'):
-            path = "%s:%s" % (self._ds._data_source, self._ds._line_number)
-        elif hasattr(self._parent._play, '_ds') and hasattr(self._parent._play._ds, '_data_source') and hasattr(self._parent._play._ds, '_line_number'):
-            path = "%s:%s" % (self._parent._play._ds._data_source, self._parent._play._ds._line_number)
-        return path
-
     def get_name(self, include_role_fqcn=True):
         ''' return the name of the task '''
 
@@ -139,7 +123,7 @@ class Task(Base, Conditional, Taggable, CollectionSearch):
             return ds
         elif isinstance(ds, dict):
             buf = ""
-            for (k, v) in iteritems(ds):
+            for (k, v) in ds.items():
                 if k.startswith('_'):
                     continue
                 buf = buf + "%s=%s " % (k, v)
@@ -153,7 +137,7 @@ class Task(Base, Conditional, Taggable, CollectionSearch):
 
     def __repr__(self):
         ''' returns a human readable representation of the task '''
-        if self.get_name() in C._ACTION_META:
+        if self.action in C._ACTION_META:
             return "TASK: meta (%s)" % self.args['_raw_params']
         else:
             return "TASK: %s" % self.get_name()
@@ -161,7 +145,7 @@ class Task(Base, Conditional, Taggable, CollectionSearch):
     def _preprocess_with_loop(self, ds, new_ds, k, v):
         ''' take a lookup plugin name and store it correctly '''
 
-        loop_name = k.replace("with_", "")
+        loop_name = k.removeprefix("with_")
         if new_ds.get('loop') is not None or new_ds.get('loop_with') is not None:
             raise AnsibleError("duplicate loop in task: %s" % loop_name, obj=ds)
         if v is None:
@@ -197,7 +181,7 @@ class Task(Base, Conditional, Taggable, CollectionSearch):
         else:
             # Validate this untemplated field early on to guarantee we are dealing with a list.
             # This is also done in CollectionSearch._load_collections() but this runs before that call.
-            collections_list = self.get_validated_value('collections', self._collections, collections_list, None)
+            collections_list = self.get_validated_value('collections', self.fattributes.get('collections'), collections_list, None)
 
         if default_collection and not self._role:  # FIXME: and not a collections role
             if collections_list:
@@ -226,7 +210,7 @@ class Task(Base, Conditional, Taggable, CollectionSearch):
             # But if it wasn't, we can add the yaml object now to get more detail
             raise AnsibleParserError(to_native(e), obj=ds, orig_exc=e)
         else:
-            self._ansible_internal_redirect_list = args_parser.internal_redirect_list[:]
+            self.resolved_action = args_parser.resolved_action
 
         # the command/shell/script modules used to support the `cmd` arg,
         # which corresponds to what we now call _raw_params, so move that
@@ -252,27 +236,17 @@ class Task(Base, Conditional, Taggable, CollectionSearch):
         else:
             new_ds['vars'] = dict()
 
-        for (k, v) in iteritems(ds):
+        for (k, v) in ds.items():
             if k in ('action', 'local_action', 'args', 'delegate_to') or k == action or k == 'shell':
                 # we don't want to re-assign these values, which were determined by the ModuleArgsParser() above
                 continue
-            elif k.startswith('with_') and k.replace("with_", "") in lookup_loader:
+            elif k.startswith('with_') and k.removeprefix("with_") in lookup_loader:
                 # transform into loop property
                 self._preprocess_with_loop(ds, new_ds, k, v)
+            elif C.INVALID_TASK_ATTRIBUTE_FAILED or k in self.fattributes:
+                new_ds[k] = v
             else:
-                # pre-2.0 syntax allowed variables for include statements at the top level of the task,
-                # so we move those into the 'vars' dictionary here, and show a deprecation message
-                # as we will remove this at some point in the future.
-                if action in C._ACTION_INCLUDE and k not in self._valid_attrs and k not in self.DEPRECATED_ATTRIBUTES:
-                    display.deprecated("Specifying include variables at the top-level of the task is deprecated."
-                                       " Please see:\nhttps://docs.ansible.com/ansible/playbooks_roles.html#task-include-files-and-encouraging-reuse\n\n"
-                                       " for currently supported syntax regarding included files and variables",
-                                       version="2.12", collection_name='ansible.builtin')
-                    new_ds['vars'][k] = v
-                elif C.INVALID_TASK_ATTRIBUTE_FAILED or k in self._valid_attrs:
-                    new_ds[k] = v
-                else:
-                    display.warning("Ignoring invalid attribute: %s" % k)
+                display.warning("Ignoring invalid attribute: %s" % k)
 
         return super(Task, self).preprocess_data(new_ds)
 
@@ -293,6 +267,14 @@ class Task(Base, Conditional, Taggable, CollectionSearch):
             e.message += '\nThis error can be suppressed as a warning using the "invalid_task_attribute_failed" configuration'
             raise e
 
+    def _validate_changed_when(self, attr, name, value):
+        if not isinstance(value, list):
+            setattr(self, name, [value])
+
+    def _validate_failed_when(self, attr, name, value):
+        if not isinstance(value, list):
+            setattr(self, name, [value])
+
     def post_validate(self, templar):
         '''
         Override of base class post_validate, to also do final validation on
@@ -340,7 +322,7 @@ class Task(Base, Conditional, Taggable, CollectionSearch):
                     else:
                         isdict = templar.template(env_item, convert_bare=False)
                         if isinstance(isdict, dict):
-                            env.update(isdict)
+                            env |= isdict
                         else:
                             display.warning("could not parse environment value, skipping: %s" % value)
 
@@ -379,9 +361,9 @@ class Task(Base, Conditional, Taggable, CollectionSearch):
     def get_vars(self):
         all_vars = dict()
         if self._parent:
-            all_vars.update(self._parent.get_vars())
+            all_vars |= self._parent.get_vars()
 
-        all_vars.update(self.vars)
+        all_vars |= self.vars
 
         if 'tags' in all_vars:
             del all_vars['tags']
@@ -393,17 +375,14 @@ class Task(Base, Conditional, Taggable, CollectionSearch):
     def get_include_params(self):
         all_vars = dict()
         if self._parent:
-            all_vars.update(self._parent.get_include_params())
+            all_vars |= self._parent.get_include_params()
         if self.action in C._ACTION_ALL_INCLUDES:
-            all_vars.update(self.vars)
+            all_vars |= self.vars
         return all_vars
 
     def copy(self, exclude_parent=False, exclude_tasks=False):
         new_me = super(Task, self).copy()
 
-        # if the task has an associated list of candidate names, copy it to the new object too
-        new_me._ansible_internal_redirect_list = self._ansible_internal_redirect_list[:]
-
         new_me._parent = None
         if self._parent and not exclude_parent:
             new_me._parent = self._parent.copy(exclude_tasks=exclude_tasks)
@@ -413,6 +392,8 @@ class Task(Base, Conditional, Taggable, CollectionSearch):
             new_me._role = self._role
 
         new_me.implicit = self.implicit
+        new_me.resolved_action = self.resolved_action
+        new_me._uuid = self._uuid
 
         return new_me
 
@@ -427,10 +408,8 @@ class Task(Base, Conditional, Taggable, CollectionSearch):
             if self._role:
                 data['role'] = self._role.serialize()
 
-            if self._ansible_internal_redirect_list:
-                data['_ansible_internal_redirect_list'] = self._ansible_internal_redirect_list[:]
-
             data['implicit'] = self.implicit
+            data['resolved_action'] = self.resolved_action
 
         return data
 
@@ -460,9 +439,8 @@ class Task(Base, Conditional, Taggable, CollectionSearch):
             self._role = r
             del data['role']
 
-        self._ansible_internal_redirect_list = data.get('_ansible_internal_redirect_list', [])
-
         self.implicit = data.get('implicit', False)
+        self.resolved_action = data.get('resolved_action')
 
         super(Task, self).deserialize(data)
 
@@ -478,15 +456,22 @@ class Task(Base, Conditional, Taggable, CollectionSearch):
         if self._parent:
             self._parent.set_loader(loader)
 
-    def _get_parent_attribute(self, attr, extend=False, prepend=False):
+    def _get_parent_attribute(self, attr, omit=False):
         '''
         Generic logic to get the attribute or parent attribute for a task value.
         '''
+        fattr = self.fattributes[attr]
+
+        extend = fattr.extend
+        prepend = fattr.prepend
 
-        extend = self._valid_attrs[attr].extend
-        prepend = self._valid_attrs[attr].prepend
         try:
-            value = self._attributes[attr]
+            # omit self, and only get parent values
+            if omit:
+                value = Sentinel
+            else:
+                value = getattr(self, f'_{attr}', Sentinel)
+
             # If parent is static, we can grab attrs from the parent
             # otherwise, defer to the grandparent
             if getattr(self._parent, 'statically_loaded', True):
@@ -500,7 +485,7 @@ class Task(Base, Conditional, Taggable, CollectionSearch):
                     if attr != 'vars' and hasattr(_parent, '_get_parent_attribute'):
                         parent_value = _parent._get_parent_attribute(attr)
                     else:
-                        parent_value = _parent._attributes.get(attr, Sentinel)
+                        parent_value = getattr(_parent, f'_{attr}', Sentinel)
 
                     if extend:
                         value = self._extend_value(value, parent_value, prepend)
@@ -511,31 +496,6 @@ class Task(Base, Conditional, Taggable, CollectionSearch):
 
         return value
 
-    def get_dep_chain(self):
-        if self._parent:
-            return self._parent.get_dep_chain()
-        else:
-            return None
-
-    def get_search_path(self):
-        '''
-        Return the list of paths you should search for files, in order.
-        This follows role/playbook dependency chain.
-        '''
-        path_stack = []
-
-        dep_chain = self.get_dep_chain()
-        # inside role: add the dependency chain from current to dependent
-        if dep_chain:
-            path_stack.extend(reversed([x._role_path for x in dep_chain]))
-
-        # add path of task itself, unless it is already in the list
-        task_dir = os.path.dirname(self.get_path())
-        if task_dir not in path_stack:
-            path_stack.append(task_dir)
-
-        return path_stack
-
     def all_parents_static(self):
         if self._parent:
             return self._parent.all_parents_static()
@@ -548,3 +508,9 @@ class Task(Base, Conditional, Taggable, CollectionSearch):
                 return self._parent
             return self._parent.get_first_parent_include()
         return None
+
+    def get_play(self):
+        parent = self._parent
+        while not isinstance(parent, Block):
+            parent = parent._parent
+        return parent._play
