@@ -23,10 +23,14 @@ import time
 import functools
 import shlex
 import typing as t
+import warnings
 
 from struct import unpack, pack
 from termios import TIOCGWINSZ
 
+# CAUTION: Avoid third-party imports in this module whenever possible.
+#          Any third-party imports occurring here will result in an error if they are vendored by ansible-core.
+
 try:
     from typing_extensions import TypeGuard  # TypeGuard was added in Python 3.10
 except ImportError:
@@ -70,14 +74,12 @@ ANSIBLE_TEST_ROOT = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
 
 # assume running from install
 ANSIBLE_ROOT = os.path.dirname(ANSIBLE_TEST_ROOT)
-ANSIBLE_BIN_PATH = os.path.dirname(os.path.abspath(sys.argv[0]))
 ANSIBLE_LIB_ROOT = os.path.join(ANSIBLE_ROOT, 'ansible')
 ANSIBLE_SOURCE_ROOT = None
 
 if not os.path.exists(ANSIBLE_LIB_ROOT):
     # running from source
     ANSIBLE_ROOT = os.path.dirname(os.path.dirname(os.path.dirname(ANSIBLE_TEST_ROOT)))
-    ANSIBLE_BIN_PATH = os.path.join(ANSIBLE_ROOT, 'bin')
     ANSIBLE_LIB_ROOT = os.path.join(ANSIBLE_ROOT, 'lib', 'ansible')
     ANSIBLE_SOURCE_ROOT = ANSIBLE_ROOT
 
@@ -129,6 +131,7 @@ class Architecture:
     Normalized architecture names.
     These are the architectures supported by ansible-test, such as when provisioning remote instances.
     """
+
     X86_64 = 'x86_64'
     AARCH64 = 'aarch64'
 
@@ -338,6 +341,17 @@ def get_ansible_version() -> str:
     return ansible_version
 
 
+def _enable_vendoring() -> None:
+    """Enable support for loading Python packages vendored by ansible-core."""
+    # Load the vendoring code by file path, since ansible may not be in our sys.path.
+    # Convert warnings into errors, to avoid problems from surfacing later.
+
+    with warnings.catch_warnings():
+        warnings.filterwarnings('error')
+
+        load_module(os.path.join(ANSIBLE_LIB_ROOT, '_vendor', '__init__.py'), 'ansible_vendor')
+
+
 @cache
 def get_available_python_versions() -> dict[str, str]:
     """Return a dictionary indicating which supported Python versions are available."""
@@ -422,7 +436,7 @@ def raw_command(
     display.info(f'{description}: {escaped_cmd}', verbosity=cmd_verbosity, truncate=True)
     display.info('Working directory: %s' % cwd, verbosity=2)
 
-    program = find_executable(cmd[0], cwd=cwd, path=env['PATH'], required='warning')
+    program = find_executable(cmd[0], cwd=cwd, path=env['PATH'], required=False)
 
     if program:
         display.info('Program found: %s' % program, verbosity=2)
@@ -553,6 +567,7 @@ def communicate_with_process(
 
 class WriterThread(WrappedThread):
     """Thread to write data to stdin of a subprocess."""
+
     def __init__(self, handle: t.IO[bytes], data: bytes) -> None:
         super().__init__(self._run)
 
@@ -570,6 +585,7 @@ class WriterThread(WrappedThread):
 
 class ReaderThread(WrappedThread, metaclass=abc.ABCMeta):
     """Thread to read stdout from a subprocess."""
+
     def __init__(self, handle: t.IO[bytes], buffer: t.BinaryIO) -> None:
         super().__init__(self._run)
 
@@ -584,6 +600,7 @@ class ReaderThread(WrappedThread, metaclass=abc.ABCMeta):
 
 class CaptureThread(ReaderThread):
     """Thread to capture stdout from a subprocess into a buffer."""
+
     def _run(self) -> None:
         """Workload to run on a thread."""
         src = self.handle
@@ -598,6 +615,7 @@ class CaptureThread(ReaderThread):
 
 class OutputThread(ReaderThread):
     """Thread to pass stdout from a subprocess to stdout."""
+
     def _run(self) -> None:
         """Workload to run on a thread."""
         src = self.handle
@@ -778,6 +796,7 @@ def generate_password() -> str:
 
 class Display:
     """Manages color console output."""
+
     clear = '\033[0m'
     red = '\033[31m'
     green = '\033[32m'
@@ -886,6 +905,7 @@ class Display:
 
 class InternalError(Exception):
     """An unhandled internal error indicating a bug in the code."""
+
     def __init__(self, message: str) -> None:
         super().__init__(f'An internal error has occurred in ansible-test: {message}')
 
@@ -898,8 +918,13 @@ class ApplicationWarning(Exception):
     """General application warning which interrupts normal program flow."""
 
 
+class TimeoutExpiredError(SystemExit):
+    """Error raised when the test timeout has been reached or exceeded."""
+
+
 class SubprocessError(ApplicationError):
     """Error resulting from failed subprocess execution."""
+
     def __init__(
         self,
         cmd: list[str],
@@ -936,6 +961,7 @@ class SubprocessError(ApplicationError):
 
 class MissingEnvironmentVariable(ApplicationError):
     """Error caused by missing environment variable."""
+
     def __init__(self, name: str) -> None:
         super().__init__('Missing environment variable: %s' % name)
 
@@ -948,6 +974,7 @@ class HostConnectionError(ApplicationError):
     Raised by provisioning code when one or more provisioning threads raise this exception.
     Also raised when an SSH connection fails for the shell command.
     """
+
     def __init__(self, message: str, callback: t.Callable[[], None] = None) -> None:
         super().__init__(message)
 
@@ -1144,3 +1171,5 @@ def type_guard(sequence: c.Sequence[t.Any], guard_type: t.Type[C]) -> TypeGuard[
 
 
 display = Display()  # pylint: disable=locally-disabled, invalid-name
+
+_enable_vendoring()
